local Loveable=require 'Loveable'

local MAXFUEL=1000

local M=class(Loveable, function(self, parent, W, H)
  Loveable.init(self)
  self.parent=parent
  self.X, self.Y=20,700
  self.vx, self.vy=0,0
  self.ax, self.ay=0,0
  self.DX=0
  self.H=H
  self.Color={0,0,120}
  self.landed=false
  self.fuel=MAXFUEL
  self.scores=0
end)

function M:setLanded(l, px)
  if l then
    if self.landed then return end
    LASTPLATFORM=LASTPLATFORM or 0
    if px>LASTPLATFORM then
      self.scores=self.scores+(px-LASTPLATFORM)+self.fuel
      LASTPLATFORM=px
    end
    self.landed=true
    self.vx=0
    self.vy=0
    self.ax=0
    self.ay=0
    self.fuel=MAXFUEL
  else
    self.landed=false
  end
end

function M:setSize(W, H)
  self.W, self.H=W, H
  return self
end

local isDown = love.keyboard.isDown
function M:update(dt)
  if self.paused then return end
  if self._crashed then return end

  if self.landed then
    self.ax=0
    self.ay=0
    if isDown('down') then
      self:setLanded(false)
      self.ay=1.2
    end
  else
    if self.fuel>0 then
      if isDown('right') then
        self.ax=0.04
        self.fuel=self.fuel-1
      elseif isDown('left') then
        self.ax=-0.04
        self.fuel=self.fuel-1
      else
        self.ax=0
      end
      if isDown('up') then
        self.ay=0.1
        self.fuel=self.fuel-4
      else
        self.ay=-0.02
      end
    else
      self.ax=0
      self.ay=-0.02
    end
  end

  if self.ay>0 or self.ax~=0 then
    if not self.firing then
      self.parent.Sounds:fireStart()
    end
    self.firing=true
  else
    if self.firing then
      self.parent.Sounds:fireStop()
    end
    self.firing=false
  end
  self.vx=self.vx+self.ax
  self.vy=self.vy+self.ay

  self.X=self.X+self.vx
  self.Y=self.Y+self.vy

  if self.vx>10 then self.vx=10 end
  if self.vx<-10 then self.vx=-10 end
  if self.vy>10 then self.vy=10 end
  if self.vy<-10 then self.vy=-10 end

  if self.X<self.DX then 
    self.X=self.DX
    self.vx=0
  end
  if self.X>self.DX+self.W-250 then
    self.parent:changeOffset(math.max(2, self.X-(self.DX+self.W)+30))
  end

  if self.Y<0 then
    self.Y=0
    self.vy=0
  end
  if self.Y>self.H then 
    self.Y=self.H
    self.vy=0
  end
    self:checkHit()
end


function M:checkHit()
  if self.debug then
    self.MARKS={}
  end
  local ii,x=1,0
  local T=self.parent.Mountains.T
  for i=1,#T, 2 do
    if T[i]>=self.X then
      ii=i-2
      break
    end
  end
  local ok=true
  for b=1,4,2 do
    if T[b+ii] then
      if T[b+ii]<=self.H-self.Y then
        ok=false
      end
      if self.debug then
        self.MARKS[#self.MARKS+1]={T[b+ii-1], T[b+ii]}
      end
    end
  end
  if ok then self.Color={0,0,255} else 
    self:crashed()
  end
end

function M:crashed()
  self.Color={255,0,0}
  self._crashed=true
  self.ax=0
  self.ay=0
  LASTPLATFORM=self.X
  if self.firing then
    self.firing=false
    self.parent.Sounds:fireStop()
  end
  self.parent.Sounds:crash()
end

function M:drawBareLander(x, y)
  love.graphics.polygon('fill', x-10,y, x-8,y-3, x-5,y-5, x+5,y-5, x+8,y-3, x+10,y, x+8,y+4, x-8,y+4)
  love.graphics.setLineWidth(2)
  love.graphics.line(x-10,y+10, x-2,y+4)
  love.graphics.line(x+10,y+10, x+2,y+4)
  love.graphics.line(x,y+10, x,y+4)
  love.graphics.setLineWidth(3)
  love.graphics.circle('fill', x-10,y+10, 2, 5)
  love.graphics.circle('fill', x+10,y+10, 2, 5)
end

function M:draw()
    love.graphics.translate(-self.DX, 0)
  -- draw colored square to framebuffer
    local x=self.X
    local y=self.H-self.Y
    love.graphics.setLineWidth(3)
    if self._crashed then
      local c=self._crashcounter or 0
      if c<255 then 
        c=c+1
        self._crashcounter=c
      end

      love.graphics.setColor(255,0,0,255-c)
      love.graphics.circle('fill', x,y, 10+c,50)
      if c>20 then
        love.graphics.setColor(255,255,0,255-c)
        love.graphics.circle('fill', x,y, c-10,50)
      end

    else
      love.graphics.setColor(unpack(self.Color))
    end

    self:drawBareLander(x, y)

    love.graphics.setColor(255,100,100,255)
    if self.ay>0 then
      love.graphics.triangle('fill', x-10,y+10+math.random(20,30), x,y+10, x+10, y+10+math.random(20,30))
    end
    if self.ax<0 then
      love.graphics.triangle('fill', x+10+math.random(10,13),y-10, x+10,y, x+10+math.random(10,13),y+10)
    end
    if self.ax>0 then
      love.graphics.triangle('fill', x-10-math.random(10,13),y-6, x-10,y, x-10-math.random(10,13),y+6)
    end
    if self.MARKS then
      love.graphics.setColor(200,000,0,180)
      for k,v in ipairs(self.MARKS) do
        love.graphics.circle('fill', v[1], v[2], 5, 10)
      end
    end
    love.graphics.translate(self.DX, 0)
    if self._crashed then
      love.graphics.setColor(255,000,0,255)
      love.graphics.setFont(BIGFONT)
      love.graphics.printf(("CRASHED !!! Press SPACE"), self.W/2, self.H/2,  230, 'center')
    end
    love.graphics.setColor(unpack(self.Color))
    love.graphics.setFont(SMALLFONT)
    love.graphics.printf(string.format("Lander Vx=%0.2f Vy=%0.2f", self.vx, self.vy), 10, 10, 600, 'left')
    love.graphics.printf(string.format("SCORES: %04d", self.scores), 1000, 10, 280, 'right')
    love.graphics.setColor(0,0,255,155)
    if self.fuel<=300 then
      love.graphics.setColor(255, 255, 0,155)
    end
    if self.fuel<=100 then
      love.graphics.setColor(255,0,0,155)
    end
    --love.graphics.printf(string.format("Fuel=%d", self.fuel), 10, self.H-60, 800, 'left')
    love.graphics.rectangle('fill', 10,self.H-60, self.fuel/3, 20)
end

function M:setOffset(x)
  self.DX=x
end

function M:changeOffset(dx)
  self.DX=self.DX+dx
  return self
end

function M:keypressed(a, b)
  if a=='d' then
    self.debug=not self.debug 
    if not self.debug then self.MARKS=nil end
  end
  if a==' ' then
    local px=self.parent.Mountains.PX
    local py=self.parent.Mountains.PY
    self._crashcounter=nil
    if px and self.X>px then
      self.X=px+40
      self.Y=py+23
      self:setLanded(true, 0)
    else
      self.X=self.DX+20
      self.Y=700
      self.vx=0.4
      self.vy=0
      self.fuel=MAXFUEL
    end
    self._crashed=nil
    self.scores=0
  end
end

return M
