local Loveable=require 'Loveable'

local M=class(Loveable, function(self, parent)
  Loveable.init(self)
  self.parent=parent
  self.PX, self.PY=nil, nil
  self.PW=90
  self.cached=nil

  math.randomseed(os.time())
  self:setSize(1000, 800)
  self:setLimits(50, 600)
  self.T={}
  self:setOffset(0)
  self.fb=love.graphics.newFramebuffer()
end)

function M:setSize(W, H)
  self.W, self.H=W, H
  if self.T then
    self.PX, self.PY=nil, nil
    while self:generate() do end
  end
  return self
end

function M:setOffset(x)
  self.DX=x or 0
  while self:generate() do end
  self.cached=nil
  return self
end

function M:changeOffset(dx)
  self.DX=self.DX+dx
  if self.PX and self.DX>self.PX+self.PW then
    self.PX=nil
  end
  while self:generate() do end
  self.cached=nil
  return self
end

function M:setLimits(ymin, ymax)
  self.YMIN=ymin
  self.YMAX=ymax
end
rnd=math.random

function M:generate()
  if #self.T==0 then
    self.T={0, self.H-100} --firs point
  end
  local lastX=self.T[#self.T-1] or 0
  local lastY=self.H-self.T[#self.T] or 0
  local desiredX=self.DX+self.W*1.5
  local dx, dy
  if lastX<=desiredX then
    if not self.PX and rnd(0,100)<5 then --platform
      self.PX=lastX
      self.PY=lastY
      dx=self.PW
      dy=0
    else
      dx=rnd(5, 50)
      dy=rnd(-100,100)
    end
    lastX=lastX+dx
    lastY=lastY+dy
    if lastY>self.YMAX then lastY=self.YMAX end
    if lastY<self.YMIN then lastY=self.YMIN end

    self.T[#self.T+1]=lastX
    self.T[#self.T+1]=self.H-lastY
  end
  if lastX<=desiredX then
    return true
  else
  end
end

function M:prune()
  local x2=self.T[3]
  while x2<self.DX do
    table.remove(self.T, 1)
    table.remove(self.T, 1)
    x2=self.T[3]
  self.cached=nil
  end
end

function M:load()
end

local isDown = love.keyboard.isDown
function M:update(dt)
  if self.PX then
    local l=self.parent.Lander
    local x=self.parent.Lander.X
    local y=self.parent.Lander.Y
    local vx=math.abs(self.parent.Lander.vx)
    local vy=math.abs(self.parent.Lander.vy)
    local dx=self.PX-x
    local dy=self.PY-y
    if dx<=-10 and dx>=-80 and dy<=-20 and dy>=-21 then
      if vy>1.5 then
        l:crashed()
      else
        l:setLanded(true, self.PX)
      end
    end
  end
  if isDown('c') then 
    self.parent:changeOffset(3)
  end
  self:prune()
end

function M:draw()
  if not self.cached then
    self.cached=true
    love.graphics.setRenderTarget(self.fb)

    love.graphics.translate(-self.DX, 0)
  -- draw colored square to framebuffer
    love.graphics.setLineWidth(3)
    love.graphics.setColor(230,240,120)
    local T=self.T
    for k=1,#self.T-2,2 do
      local x1=T[k]
      local y1=T[k+1]
      local x2=T[k+2]
      local y2=T[k+3]
      --love.graphics.polygon('fill', x1, self.H, x1,self.H-y1, x2,self.H-y2, x2, self.H)
      --love.graphics.polygon('fill', x1, self.H, x1,y1, x2,y2, x2, self.H)
      love.graphics.polygon('fill', x1, 0, x1,y1, x2,y2, x2, 0)
    end
    love.graphics.setColor(130,140,20)
    love.graphics.line(self.T)

    if self.PX then
      love.graphics.setLineWidth(55)
      love.graphics.setColor(0, 230,0)
      --love.graphics.line(self.PX, self.H-self.PY, self.PX+self.PW, self.H-self.PY)
      love.graphics.rectangle('fill', self.PX, self.H-self.PY-10, self.PW, 10)
    end
    love.graphics.translate(self.DX, 0)
    love.graphics.setRenderTarget()
  end
  love.graphics.setColor(255,255,255,255)
  love.graphics.draw(self.fb,0,0,0)
end

return M
